from panda3d.core import *
load_prc_file_data("", "gl-version 3 1")
load_prc_file_data("", "basic-shaders-only #f")
load_prc_file_data("", "gl-check-errors True        \
gl-debug True     \
gl-debug-object-labels True \
gl-force-fbo-color false")
load_prc_file_data('','textures-power-2 None')
#load_prc_file_data("", "notify-level-glgsg debug")
from direct.showbase.ShowBase import ShowBase
from direct.interval.LerpInterval import LerpPosInterval,LerpHprInterval
from direct.interval.IntervalGlobal import *



class Main(ShowBase):

    def __init__(self):

        ShowBase.__init__(self)
        print(100*"*","\n")
        print("Panda version: ",PandaSystem.getVersionString())
        print(100*"*","\n")
        light = DirectionalLight("MyDirectional")
        light.color = (0.0,0.7,0.3,1.0)
        self._light_np = base.render.attach_new_node(light)
        self._light_np.set_pos(0,-500,10.0)
        self._light_np.node().set_shadow_caster(True,512,512,1)
        self._light_np.node().get_lens().set_film_size(500.0,500.0)
        self._light_np.node().get_lens().set_near_far(1.0,1000.0)
        base.render.set_light(self._light_np)

        light2 = Spotlight("MySpot")
        light2.color = (5.0,0.1,0.0,1.0)
        light2.max_distance = 8000.0
        self._light2_np = base.render.attach_new_node(light2)
        self._light2_np.set_pos(0,0,25)
        self._light2_np.node().set_shadow_caster(True,512,512,2)
        base.render.set_light(self._light2_np)

        self._np = base.loader.load_model(Filename.from_os_specific("box"))
        self._np.reparent_to(base.render)
        self._np.set_two_sided(False)

        for x in range (0,5):
            for y in range(0,5):
                for z in range(0,5):
                    np2 = base.loader.load_model(Filename.from_os_specific("box"))
                    np2.reparent_to(base.render)
                    np2.set_two_sided(False)
                    np2.set_pos(2*x*15,-10.0*y*2,-30.0*z*2)
                    np2.set_scale(15.0)

        self._np.set_shader_auto(False)
        self._light2_np.look_at(self._np)
        self._light_np.look_at(self._np)

        self._tex_array = Texture("volume")
        self._tex_array.setup_2d_texture_array(512, 512, 4, Texture.T_float, Texture.F_depth_component32)
        self._tex_array.set_clear_color( LVecBase4(0.0,0.0,0.0,1.0 ))
        self._state = SamplerState()
        self._state.setMinfilter( SamplerState.FT_linear)
        self._state.setMagfilter( SamplerState.FT_linear )
        self._state.set_wrap_u( SamplerState.WM_clamp )
        self._state.set_wrap_v( SamplerState.WM_clamp )
        self._state.set_wrap_w( SamplerState.WM_clamp )
        self._tex_array.setDefaultSampler( self._state )

        self._start_pos = self._light2_np.get_pos()
        self._end_pos = Point3(Vec3(self._start_pos.get_x()-10.0,self._start_pos.get_y(),self._start_pos.get_z()+50.0) ) 
        interval1 = LerpPosInterval(self._light2_np,5.0,pos = self._end_pos,startPos = self._start_pos) 
        interval2 = LerpPosInterval(self._light2_np,5.0,pos = self._start_pos)
        self._seq = Sequence(interval1, interval2,name="MySequence")
        self._seq.loop()
        base.taskMgr.add(self._capture)


    def _capture(self,task):
        sBuffer = self._light_np.node().getShadowBuffer(GraphicsStateGuardianBase.getGsg(0))
        sBuffer2 = self._light2_np.node().getShadowBuffer(GraphicsStateGuardianBase.getGsg(0))
        if sBuffer is not None and sBuffer2 is not None:
            self.cardmaker = CardMaker("cubemaker")
            self.cardmaker.setFrame(-1,1,-1,1)
            for layer in range(3):
                if layer in [0,1]: 
                    self.cardmaker.setUvRange((0, 1, 1, 0), (0, 0, 1, 1),(layer, layer, layer, layer))
                    self._create_card_2D(layer,0)
            #Light1
            sBuffer.addRenderTexture(self._tex_array, GraphicsOutput.RTM_bind_layered, GraphicsOutput.RTP_depth)
            attr = self._generate_light_shader(0)
            state = self._light_np.node().get_initial_state()
            state = state.add_attrib(attr,1)
            self._light_np.node().set_initial_state(state)

            #sBuffer.addRenderTexture(self._tex_array, GraphicsOutput.RTM_bind_or_copy, GraphicsOutput.RTP_depth)
            #for dr in sBuffer.get_display_regions():
            #    print("SBuffer",dr.active)
            #    dr.set_target_tex_page(0)

            #Light2
            sBuffer2.addRenderTexture(self._tex_array, GraphicsOutput.RTM_bind_layered, GraphicsOutput.RTP_depth)
            for dr in sBuffer2.get_display_regions():
                dr.disable_clears()
            sBuffer2.disable_clears()
            attr = self._generate_light_shader(1)
            state = self._light2_np.node().get_initial_state()
            state = state.add_attrib(attr,1)
            self._light2_np.node().set_initial_state(state)

            #sBuffer2.add_render_texture(self._tex_array, GraphicsOutput.RTM_bind_or_copy, GraphicsOutput.RTP_depth)
            #for dr in sBuffer2.get_display_regions():
            #    print("SBuffer2",dr.active)
            #    dr.set_target_tex_page(0)
            #for dr in sBuffer2.get_display_regions():
            #    dr.disable_clears()
            #sBuffer2.disable_clears()
            return task.done
        return task.again

    def _generate_light_shader(self,layer):
        """2D textures"""
        file = open("LightShaders/vertex_light_VS.glsl")
        vs = file.read()
        file = open("LightShaders/fragment_light_FS.glsl")
        fs = file.read()
        shader =  Shader.make(Shader.SL_GLSL, vertex = vs, fragment = fs)
        shader.set_filename(0,"Light Shader")
        attr = ShaderAttrib.make(shader)
        attr = attr.set_shader_input("DMKlayer",layer)
        return attr

    def _create_card_2D(self,layer,shift):
        card = NodePath(self.cardmaker.generate())
        card.set_two_sided(True)
        card.set_scale(0.1)           
        card.set_pos(-0.9 + 0.25*(layer+shift),0.0,0.6)
        shader = self._generate_card_2D_shader()
        card.set_shader(shader)
        card.set_shader_input("texture_array",self._tex_array)
        card.set_shader_input("DMKlayer",layer)
        card.reparent_to(base.render2d)

    def _generate_card_2D_shader(self): 
        """Debug shader for 2D texture"""   
        file = open("DebugShaders/vertex_VS.glsl")
        vs = file.read()
        file = open("DebugShaders/fragment_FS.glsl")
        fs = file.read()
        return Shader.make(Shader.SL_GLSL, vertex = vs, fragment = fs)

a = Main()
a.run()

